#import <Foundation/Foundation.h>

@protocol MYMessageServerProtocol

- (void)addMessageClient:(id)aClient;
- (BOOL)removeMessageClient:(id)aClient;
- (void)broadcastMessageString:(NSString *)aString;

@end

@interface MYMessageServer : NSObject <MYMessageServerProtocol>
{
  NSMutableArray      *_myListOfClients;
}

@end


int main (int argc, const char * argv[])
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
  MYMessageServer   *server = [[MYMessageServer alloc] init];
  NSConnection      *defaultConnection;

  defaultConnection = [NSConnection defaultConnection];
  [[NSNotificationCenter defaultCenter] addObserver:server
      selector:@selector(connectionDidDie:)
      name:NSConnectionDidDieNotification
      object:nil];
  [defaultConnection setRootObject:server];
  if ([defaultConnection registerName:@"server"] == NO)
  {
    NSLog(@"Error registering server");
  }

  [[NSRunLoop currentRunLoop] configureAsServer];
  [[NSRunLoop currentRunLoop] run];
  
  [pool release];
  return 0;
}

@implementation MYMessageServer

- (void)connectionDidDie:(NSNotification *)aNotification
{
  NSConnection     *deadConnection = [aNotification object];
  int              i;
  
  NSLog(@"Connection to client is broken.");

  // remove proxies with dead connections from client list
  for(i = [_myListOfClients count] - 1; i >= 0; i--)
  {
    id    currentClient = [_myListOfClients objectAtIndex:i];

    NS_DURING
      if([currentClient respondsToSelector:@selector(connectionForProxy)])
      {
        if(deadConnection == [currentClient connectionForProxy])
        {
          // remove proxy with dead connection
          [_myListOfClients removeObjectAtIndex:i];
          NSLog(@"Removed client from client list.");
        }
      }
    NS_HANDLER
      [_myListOfClients removeObjectAtIndex:i];
      NSLog(@"Removed client from client list.");
    NS_ENDHANDLER
  }
}

- (void)addMessageClient:(id)aClient
{
  if(nil == _myListOfClients)
  {
    _myListOfClients = [[NSMutableArray alloc] init];
  }
  [_myListOfClients addObject:aClient];
  NSLog(@"Added client");
}

- (BOOL)removeMessageClient:(id)aClient
{
  [_myListOfClients removeObject:aClient];
  NSLog(@"Removed client");

  return YES;
}

- (void)broadcastMessageString:(NSString *)aString
{
  [_myListOfClients makeObjectsPerformSelector:@selector(appendMessageString:)
      withObject:aString];
}


- (void)dealloc
{
  [_myListOfClients release];
  _myListOfClients = nil;

  [super dealloc];
}


@end

